
// Copyright (c) 2015, Tobias Mueller tm(at)tm3d.de
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the
//    distribution.
//  * All advertising materials mentioning features or use of this
//    software must display the following acknowledgement: This product
//    includes software developed by tm3d.de and its contributors.
//  * Neither the name of tm3d.de nor the names of its contributors may
//    be used to endorse or promote products derived from this software
//    without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "../common/I2C/USI_TWI_Master.h"
#include "../common/I2C/MAX44009.h"
#include "../common/I2C/SHT2x.h"


extern void OWINIT();



uint8_t owid1[8]={0x28, 0xA3, 0xD9, 0x84, 0x00, 0x16, 0x05, 0x18};/**/
uint8_t owid2[8]={0x26, 0xA2, 0xD9, 0x84, 0x00, 0x00, 0x01, 0x77};/**/
uint8_t config_info1[16]={0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00};	
uint8_t config_info2[16]={0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00};	
	
#if (owid>128) 
#error "Variable not correct"
#endif

extern uint8_t mode;
extern uint8_t gcontrol;
extern uint8_t reset_indicator;
extern uint8_t alarmflag;
volatile uint8_t wdcounter;


#if  defined(__AVR_ATtiny24__)||defined(__AVR_ATtiny44__)  || defined(__AVR_ATtiny84__) || defined(__AVR_ATtiny24A__)||defined(__AVR_ATtiny44A__)  || defined(__AVR_ATtiny84A__)
ISR(WATCHDOG_vect) {
#else
ISR(WDT_vect) {
#endif 
	//sleep_disable();          // Disable Sleep on Wakeup
	wdcounter++;
	if (reset_indicator==1) reset_indicator++;
	else if (reset_indicator==2) mode=0;
/*	if (timeout==2) {
		DIS_TIMER;
		EN_OWINT;
		mode=OWM_SLEEP;
	}
	timeout++;*/
	//sleep_enable();           // Enable Sleep Mode

}


typedef union {
	volatile uint8_t bytes[8];
	struct {
		uint16_t temp;  //0
		uint8_t TH;  //2
		uint8_t TL;  //3
		uint8_t config;  //4
		uint8_t rrFF; //5
		uint8_t rr00; //6
		uint8_t rr10; //7
	};
} pack1_t;
volatile pack1_t pack1;



typedef union {
	#if  defined(__AVR_ATtiny25__)
	volatile uint8_t bytes[16];
	#else
	volatile uint8_t bytes[64];
	#endif
	struct {
		uint8_t status;  //1
		int16_t temp;  //2
		uint16_t voltage;  //4
		uint16_t current;  //6
		uint8_t threshold; //8
		
		uint8_t page1[8]; //9
		#if  defined(__AVR_ATtiny25__)
		#else
		uint8_t page2[8]; //17
		uint8_t page3[8]; //25
		uint8_t page4[8];  //33
		uint8_t page5[8];  //41
		uint8_t page6[8];  //49
		uint8_t page7[8];  //57
		//uint8_t crc;  //65
		#endif
	};
} pack2_t;
volatile pack2_t pack2;




volatile int16_t am2302_temp;
volatile uint16_t am2302_hum;


uint8_t userRegister[1];
int16_t sRH,sT;
volatile double temperatureC,humidityRH;
volatile double l;


int main(void){
    PRR|=(1<<PRADC);  // adc for save Power
	pack1.temp=0x0550;
	pack1.config=0x7F;
	pack1.TH=75;
	pack1.TL=70;
	pack1.rrFF=0xFF;
	pack1.rr00=0;
	pack1.rr10=0x10;
	PORTA=0xFF;
	PORTB=0xFF;
	OWINIT();

	MCUCR &=~(1<<PUD); //All Pins Pullup...
	MCUCR |=(1<<BODS);

	WDTCSR |= ((1<<WDCE) );   // Enable the WD Change Bit//| (1<<WDE)
	WDTCSR |=   (1<<WDIE) |              // Enable WDT Interrupt
	(1<<WDP2) | (1<<WDP1);   // Set Timeout to ~1 seconds

	MCUSR=0;
	USI_TWI_Master_Initialise();
	SHT2x_SoftReset();
	SHT2x_ReadUserRegister(userRegister);
	//(userRegister & ~SHT2x_RES_MASK) | SHT2x_RES_10_13BIT;
	SHT2x_WriteUserRegister(userRegister); //write changed user reg
	// --- measure humidity with "Hold Master Mode (HM)" ---
	SHT2x_MeasurePoll(HUMIDITY, &sRH);
	// --- measure temperature with "Polling Mode" (no hold master) ---
	SHT2x_MeasurePoll(TEMP, &sT);
	//-- calculate humidity and temperature --
	temperatureC = SHT2x_CalcTemperatureC(sT);
	humidityRH = SHT2x_CalcRH(sRH);
	
	gcontrol=1;
	sei();
    while(1)   {
		if (wdcounter>0) {
			SHT2x_MeasurePoll(HUMIDITY, &sRH);
			// --- measure temperature with "Polling Mode" (no hold master) ---
			SHT2x_MeasurePoll(TEMP, &sT);
			//-- calculate humidity and temperature --
			temperatureC = SHT2x_CalcTemperatureC(sT)*10.0;
			humidityRH = SHT2x_CalcRH(sRH)*10.0;
			double hhum=(1.0546-0.000216*temperatureC)*(humidityRH);
			am2302_hum=0.318*hhum +76.0;
			am2302_temp=temperatureC*25.6;
			wdcounter=0;
		}
	
		if (gcontrol==1) {
			l=MAX44009getlux();		
			if (l<0.030) l=0.030; //Darf nicht 0 sein. minimum -35C Sensor minimum 0.045
			//double l=1000;
			l=log(l)*10*16;
			uint16_t w=l;
			uint8_t t8=w>>4;
			uint8_t af=0;
			if (t8>pack1.TH) af=1;
			if (t8<=pack1.TL) af=1; 
			cli();
			pack1.temp=w;
			//pack.temp++;
			alarmflag=af;
			sei();			
			gcontrol=0;
		}
		if ((gcontrol==2)||(gcontrol==3)) {
			gcontrol=0;
			
		}

		
#if  defined(__AVR_ATtiny25__)||defined(__AVR_ATtiny45__)  || defined(__AVR_ATtiny85__)
			if (((TIMSK & (1<<TOIE0))==0)&& (mode==0))
#endif			
#if  defined(__AVR_ATtiny24__)||defined(__AVR_ATtiny44__)  || defined(__AVR_ATtiny84__) ||defined(__AVR_ATtiny24A__)||defined(__AVR_ATtiny44A__)  || defined(__AVR_ATtiny84A__)
			if (((TIMSK0 & (1<<TOIE0))==0)&& (mode==0))
#endif
			  {

			MCUCR|=(1<<SE)|(1<<SM1);
			MCUCR&=~(1<<ISC01);
		} else {
			MCUCR|=(1<<SE);
			MCUCR&=~(1<<SM1);
		}
		//MCUCR&=~(1<<ISC01);
		asm("SLEEP");
   }


}